'use strict';

/**
 * `input` type prompt
 */

var util = require('util');
var observe = require('../utils/events');
var utils = require('../utils/');
var Base = require('./base');

/**
 * Module exports
 */

module.exports = Prompt;

/**
 * Constructor
 */

function Prompt() {
  return Base.apply(this, arguments);
}
util.inherits(Prompt, Base);

/**
 * Start the Inquiry session
 * @param  {Function} cb      Callback when prompt is done
 * @return {this}
 */

Prompt.prototype._run = function(cb) {
  this.done = cb;

  // Once user confirm (enter key)
  var events = observe(this.rl);
  var submit = events.line.map(this.filterInput.bind(this));

  var validation = this.handleSubmitEvents(submit);
  validation.success.forEach(this.onEnd.bind(this));
  validation.error.forEach(this.onError.bind(this));

  events.keypress.takeUntil(validation.success).forEach(this.onKeypress.bind(this));

  // Init
  this.render();
  return this;
};

/**
 * Render the prompt to screen
 * @return {Prompt} self
 */

Prompt.prototype.render = function(error) {
  var cursor = 0;
  var message = this.getQuestion();

  if (this.status === 'answered') {
    message += utils.chalk.cyan(this.answer);
  } else {
    message += this.rl.line;
  }

  if (error) {
    message += '\n' + utils.chalk.red('>> ') + error;
    cursor++;
  }

  this.screen.render(message, {
    cursor: cursor
  });
};

/**
 * When user press `enter` key
 */

Prompt.prototype.filterInput = function(input) {
  if (!input) {
    return this.opt.default != null ? this.opt.default : '';
  }
  return input;
};

Prompt.prototype.onEnd = function(state) {
  this.filter(state.value, function(filteredValue) {
    this.answer = filteredValue;
    this.status = 'answered';

    // Re-render prompt
    this.render();

    this.screen.done();
    this.done(state.value);
  }.bind(this));
};

Prompt.prototype.onError = function(state) {
  this.render(state.isValid);
};

/**
 * When user press a key
 */

Prompt.prototype.onKeypress = function() {
  this.render();
};
